"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var os_1 = require("os");
var path_1 = require("path");
var logger_1 = require("./logger");
var util_functions_1 = require("./util-functions");
var DEFAULT_CONFIG = {
    blacklistPatterns: [],
    commands: [],
    copyToClipboardCommand: '',
    gitShowOptions: '--patch-with-stat --stat-width 1000 --color',
    notificationTimeout: 5000,
    searchIndexLimit: 300000,
    showLineNumbers: false,
    useLegacyEscapeKeyBehavior: false,
    useSearchIndex: true,
};
var _getConfigFilePath = function () {
    if (process.env.JACK_CLI_CONFIG_FILE_PATH) {
        if (!process.env.JACK_CLI_CONFIG_FILE_PATH.endsWith('.json')) {
            logger_1.logger.warn('The JACK_CLI_CONFIG_FILE_PATH environment variable must define a path to a *.json file. If this file does not exist, jack-cli will create it.');
            process.exit(1);
        }
        return process.env.JACK_CLI_CONFIG_FILE_PATH;
    }
    return path_1.join(os_1.homedir(), '.jack.json');
};
var _readConfig = function (configFilePath) {
    if (!fs.existsSync(configFilePath)) {
        fs.writeFileSync(configFilePath, JSON.stringify(DEFAULT_CONFIG, null, 4));
        return DEFAULT_CONFIG;
    }
    return __assign(__assign({}, DEFAULT_CONFIG), require(configFilePath));
};
var CONFIG_FILE_PATH = _getConfigFilePath();
var CONFIG = _readConfig(CONFIG_FILE_PATH);
exports.getBlacklistPatterns = function () {
    return CONFIG.blacklistPatterns.map(function (blacklistPattern) { return new RegExp(blacklistPattern); });
};
exports.getCommands = function () { return CONFIG.commands; };
exports.getConfigFilePath = function () { return CONFIG_FILE_PATH; };
exports.getCopyToClipboardCommandArgs = function () {
    return util_functions_1.stringToCommandArray(CONFIG.copyToClipboardCommand.trim());
};
exports.getGitShowOptions = function () {
    return util_functions_1.stringToCommandArray(CONFIG.gitShowOptions);
};
exports.getNotificationTimeout = function () { return CONFIG.notificationTimeout; };
exports.getSearchIndexLimit = function () { return CONFIG.searchIndexLimit; };
exports.getShowLineNumbers = function () { return CONFIG.showLineNumbers; };
exports.getUseLegacyEscapeKeyBehavior = function () {
    return CONFIG.useLegacyEscapeKeyBehavior;
};
exports.getUseSearchIndex = function () { return CONFIG.useSearchIndex; };
//# sourceMappingURL=config-util.js.map