"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var config_util_1 = require("./config-util");
var promisify_child_process_1 = require("./promisify-child-process");
exports.gitCommitMessage = function (SHA) {
    return promisify_child_process_1.spawnPromise('git', ['show', SHA, '-s', '--pretty=format:%s']);
};
exports.gitDiff = function (SHA1, SHA2) {
    return promisify_child_process_1.spawnPromise('git', [
        'diff',
        SHA1 + "^.." + SHA2,
        '--patch',
        '--stat-width=1000',
    ]);
};
exports.gitDiffNameOnly = function (SHA1, SHA2) {
    return promisify_child_process_1.spawnPromise('git', ['diff', SHA1 + "^.." + SHA2, '--name-only']);
};
exports.gitShow = function (SHA) {
    var commandArray = ['show', SHA];
    var gitShowOptions = config_util_1.getGitShowOptions();
    if (gitShowOptions) {
        commandArray = commandArray.concat(gitShowOptions);
    }
    return promisify_child_process_1.spawnPromise('git', commandArray);
};
exports.gitTopLevel = function () {
    return promisify_child_process_1.spawnPromise('git', ['rev-parse', '--show-toplevel']);
};
exports.sortSHAs = function (SHA1, SHA2) {
    return promisify_child_process_1.spawnPromise('git', ['merge-base', '--is-ancestor', SHA1, SHA2])
        .then(function () { return Promise.resolve([SHA1, SHA2]); })
        .catch(function () { return Promise.resolve([SHA2, SHA1]); });
};
//# sourceMappingURL=git-util.js.map